@@ -18,8 +18,9 @@ from distutils.spawn import find_executable
 ## 3rd party
 import numpy as np
 import pandas as pd
+import pyfastx
 from Bio import SeqIO
-from pyfasta import Fasta
+#from pyfaidx import Fasta
 from scipy.stats import truncnorm
 ## application
 from MGSIM import Utils
@@ -84,7 +85,7 @@ def sim_barcode_reads(barcodes, genome_table, abund_table, tmp_dir, args, rndSee
                          frag_size_max=int(args['--frag-size-max']))
 
         # parsing fragments
-        func = partial(parse_frags, barcode=barcode, out_dir=tmp_dir)
+        func = partial(parse_frags, barcode=barcode, out_dir=tmp_dir, tmp_dir=tmp_dir)
         frag_files = [func(refs.loc[refs['Taxon'] == taxon]) for taxon in refs['Taxon'].unique()]
         
         # simulating reads
@@ -100,6 +101,7 @@ def sim_barcode_reads(barcodes, genome_table, abund_table, tmp_dir, args, rndSee
                                        seq_depth=float(args['--seq-depth']),
                                        total_barcodes=int(float(args['--barcode-total'])),
                                        art_params=art_params,
+                                       tmp_dir=tmp_dir,
                                        debug=args['--debug'])
             read_fq_files.append(fasta_files)
             frag_tsv_files.append(tsv_file)
@@ -117,7 +119,8 @@ def format_params(d):
             dd[k] = v
     return dd
 
-def sim_illumina(frag_fasta, barcode, seq_depth, total_barcodes, art_params, debug=False):
+def sim_illumina(frag_fasta, barcode, seq_depth, total_barcodes,
+                 art_params, tmp_dir, debug=False):
     """Simulating illumina reads for each fragment from each barcode
     Parameters
     ----------
@@ -131,6 +134,8 @@ def sim_illumina(frag_fasta, barcode, seq_depth, total_barcodes, art_params, deb
         Total number of barcodes
     art_params : dict
         art_illumina parameters
+    tmp_dir : str
+        temporary directory path
     debug : bool
         debug mode 
     """    
@@ -140,11 +145,12 @@ def sim_illumina(frag_fasta, barcode, seq_depth, total_barcodes, art_params, deb
     art_params = format_params(art_params)
     
     # calculate fold coverage to simulate
-    fold = calc_fold(frag_fasta, seq_depth, total_barcodes, art_params)
+    fold = calc_fold(frag_fasta, seq_depth, total_barcodes, art_params, tmp_dir=tmp_dir)
     
     # art_illumina command
     art_params = ' '.join(['{} {}'.format(k,v) for k,v in art_params.items()])
-    cmd = 'art_illumina {art_params} --noALN --id {barcode} -f {fold} -i {input} -o {output_prefix}'
+    cmd = 'art_illumina {art_params} --noALN --id {barcode}'
+    cmd += ' -f {fold} -i {input} -o {output_prefix}'
     cmd = cmd.format(art_params=art_params,
                      barcode=barcode + '-',
                      fold=fold,
@@ -174,9 +180,16 @@ def sim_illumina(frag_fasta, barcode, seq_depth, total_barcodes, art_params, deb
         return [R0_file]
     else:
         msg = 'Cannot find art_illumina output files!'
-        raise ValueError(msg)
+        raise ValueError(msg)    
 
-def calc_fold(frag_fasta, seq_depth, total_barcodes, art_params):
+def get_total_seq_len(fasta_file):
+    """Simple function that uses pyfastx to quickly read in a fasta,
+    and then the sum of sequence lengths is returned
+    """
+    x = [len(seq) for h,seq in pyfastx.Fasta(fasta_file, build_index=False)]
+    return sum(x)
+
+def calc_fold(frag_fasta, seq_depth, total_barcodes, art_params, tmp_dir):
     """Calculate fold coverage to simulate per barcode
     Parameters
     ----------
@@ -188,11 +201,12 @@ def calc_fold(frag_fasta, seq_depth, total_barcodes, art_params):
         Total number of barcodes
     art_params : dict
         art_illumina parameters
+    tmp_dir : str
+        temp_dir
     """
     # fold coverage
     ## total length of fragments
-    f = Fasta(frag_fasta)
-    total_frag_len = sum([len(f[x]) for x in f.keys()])
+    total_frag_len = get_total_seq_len(frag_fasta)
     ## length of reads
     try:
         read_len = int(float(art_params['--len']))
@@ -207,7 +221,7 @@ def calc_fold(frag_fasta, seq_depth, total_barcodes, art_params):
     ## fold calc
     seqs_per_barcode = seq_depth / total_barcodes
     fold  = seqs_per_barcode * read_len / total_frag_len
-    
+    # return
     return fold
 
 def combine_frag_tsv(tsv_files, output_dir, debug=False):
@@ -270,13 +284,12 @@ def combine_reads(fq_files, output_dir, name_fmt='{readID} BX:Z:{barcodeID}',
 
 def fix_nucs(seq):
     ambig = {'A', 'T', 'G', 'C'}
-    seq = ['N' if x not in ambig else x for x in seq]
-    return ''.join(seq)
+    seq = ['N' if x not in ambig else x for x in seq.rstrip()]
+    return ''.join(seq) + '\n'
         
 def _combine_reads(read_files, out_dir, out_file, name_fmt, seq_depth=None):
     """Combining temporary read files.
     eg., @ST-J00101:121:HYCGGBBXX:5:1101:30533:1191 BX:Z:A58B91C07D86
-
     """
     if seq_depth is not None:
         cnt = 0
@@ -351,8 +364,15 @@ def sim_frags(refs, n_frags, frag_size_loc, frag_size_sd,
                                             loc=frag_size_loc,
                                             scale=frag_size_sd)
     return refs
+
+def read_fasta(fasta_file):
+    """Fast reading into memory of fasta file
+    return: dict
+    """
+    seqs = {h:seq for h,seq in pyfastx.Fasta(fasta_file, build_index=False)}
+    return seqs
     
-def parse_frags(refs, barcode, out_dir):
+def parse_frags(refs, barcode, out_dir, tmp_dir):
     """Parsing fragment from a genome and writing them to a file.
     Giving each simulated fragment a UUID. 
 
@@ -371,7 +391,7 @@ def parse_frags(refs, barcode, out_dir):
     taxon = regex.sub('_', taxon)
     fasta_file = refs['Fasta'].unique()[0]
     # loading fasta
-    f = Fasta(fasta_file)
+    f = read_fasta(fasta_file)
     contig_ids = list(f.keys())
     for x in contig_ids:
         assert('|' not in x)
@@ -391,7 +411,7 @@ def parse_frags(refs, barcode, out_dir):
                 if contig_len >= frag_size:
                     frag_uuid = str(uuid.uuid4()).replace('-', '')
                     frag_max_end = contig_len - frag_size
-                    assert frag_max_end >= 0
+                    frag_max_end = 1 if frag_max_end < 1 else frag_max_end
                     frag_start = np.random.randint(0, frag_max_end)
                     frag_end = frag_start + frag_size
                     ## writing sequence
